@@ -34,7 +34,7 @@ HTML_WARNINGS = ENV_WARNINGS + """\
 %(root)s/index.rst:\\d+: WARNING: unknown option: '&option'
 %(root)s/index.rst:\\d+: WARNING: citation not found: missing
 %(root)s/index.rst:\\d+: WARNING: a suitable image for html builder not found: foo.\\*
-%(root)s/index.rst:\\d+: WARNING: Could not lex literal_block as "c". Highlighting skipped.
+%(root)s/index.rst:\\d+: WARNING: Could not lex literal_block .* as "c". Highlighting skipped.
 """
 
 
@@ -60,7 +60,7 @@ def flat_dict(d):
         [
             zip(cycle([fname]), values)
             for fname, values in d.items()
-        ]
+        ],
     )
 
 
@@ -118,24 +118,22 @@ def test_html_warnings(app, warning):
     html_warnings_exp = HTML_WARNINGS % {
         'root': re.escape(app.srcdir.replace(os.sep, '/'))}
     assert re.match(html_warnings_exp + '$', html_warnings), \
-        'Warnings don\'t match:\n' + \
+        "Warnings don't match:\n" + \
         '--- Expected (regex):\n' + html_warnings_exp + \
         '--- Got:\n' + html_warnings
 
 
-@pytest.mark.sphinx('html', confoverrides={'html4_writer': True})
-def test_html4_output(app, status, warning):
-    app.build()
-
-
-def test_html4_deprecation(make_app, tempdir):
+def test_html4_error(make_app, tempdir):
     (tempdir / 'conf.py').write_text('', encoding='utf-8')
-    app = make_app(
-        buildername='html',
-        srcdir=tempdir,
-        confoverrides={'html4_writer': True},
-    )
-    assert 'HTML 4 output is deprecated and will be removed' in app._warning.getvalue()
+    with pytest.raises(
+        ConfigError,
+        match=r'HTML 4 is no longer supported by Sphinx',
+    ):
+        make_app(
+            buildername='html',
+            srcdir=tempdir,
+            confoverrides={'html4_writer': True},
+        )
 
 
 @pytest.mark.parametrize("fname,expect", flat_dict({
@@ -396,7 +394,7 @@ def test_html4_deprecation(make_app, tempdir):
     'otherext.html': [
         (".//h1", "Generated section"),
         (".//a[@href='_sources/otherext.foo.txt']", ''),
-    ]
+    ],
 }))
 @pytest.mark.sphinx('html', tags=['testtag'],
                     confoverrides={'html_context.hckey_co': 'hcval_co'})
@@ -1434,7 +1432,7 @@ def test_html_sidebar(app, status, warning):
 @pytest.mark.parametrize('fname,expect', flat_dict({
     'index.html': [(".//em/a[@href='https://example.com/man.1']", "", True),
                    (".//em/a[@href='https://example.com/ls.1']", "", True),
-                   (".//em/a[@href='https://example.com/sphinx.']", "", True)]
+                   (".//em/a[@href='https://example.com/sphinx.']", "", True)],
 
 }))
 @pytest.mark.sphinx('html', testroot='manpage_url', confoverrides={
@@ -1491,13 +1489,12 @@ def test_html_math_renderer_is_imgmath(app, status, warning):
                     confoverrides={'extensions': ['sphinxcontrib.jsmath',
                                                   'sphinx.ext.imgmath']})
 def test_html_math_renderer_is_duplicated(make_app, app_params):
-    try:
-        args, kwargs = app_params
+    args, kwargs = app_params
+    with pytest.raises(
+        ConfigError,
+        match='Many math_renderers are registered. But no math_renderer is selected.',
+    ):
         make_app(*args, **kwargs)
-        raise AssertionError()
-    except ConfigError as exc:
-        assert str(exc) == ('Many math_renderers are registered. '
-                            'But no math_renderer is selected.')
 
 
 @pytest.mark.sphinx('html', testroot='basic',
@@ -1521,12 +1518,9 @@ def test_html_math_renderer_is_chosen(app, status, warning):
                                                   'sphinx.ext.mathjax'],
                                    'html_math_renderer': 'imgmath'})
 def test_html_math_renderer_is_mismatched(make_app, app_params):
-    try:
-        args, kwargs = app_params
+    args, kwargs = app_params
+    with pytest.raises(ConfigError, match="Unknown math_renderer 'imgmath' is given."):
         make_app(*args, **kwargs)
-        raise AssertionError()
-    except ConfigError as exc:
-        assert str(exc) == "Unknown math_renderer 'imgmath' is given."
 
 
 @pytest.mark.sphinx('html', testroot='basic')
